@@ -64,6 +64,17 @@ class Linter(visitor.Visitor):
         self.double_quote_re = re.compile(r"\".+\"")
         self.ellipsis_re = re.compile(r"\.\.\.")
 
+        self.message_errors = {}
+        if "CO01" in config and config["CO01"]["enabled"]:
+            self.brand_names = config["CO01"].get("brands", [])
+        else:
+            self.brand_names = []
+        if "CO02" in config and config["CO02"]["enabled"]:
+            # Transform lowercase
+            self.banned_words = [word.lower() for word in config["CO02"]["words"]]
+        else:
+            self.banned_words = []
+
         # Syntax to ignore when checking double quotes
         self.ftl_syntax_re = [
             # Parameterized terms
@@ -82,22 +93,31 @@ class Linter(visitor.Visitor):
             # Group comments must be followed by a message. Two group comments are not
             # allowed in a row.
             "can_have_group_comment": True,
+            # If currently looking at a term
+            "is_term": False,
         }
 
         # Set this to true to debug print the root node's json. This is useful for
         # writing new lint rules, or debugging existing ones.
         self.debug_print_json = False
 
-    def check_typography(self, node):
-        def exclude_string(id, node):
-            if (
-                id in self.config
-                and node.id.name in self.config[id]["exclusions"]["messages"]
-            ):
-                return True
+    def exclude_message(self, rule, message_id, filename=None):
+        """Check if message with ID should be ignored"""
 
+        # Rule is not set in config or doesn't have exclusions
+        if rule not in self.config or "exclusions" not in self.config[rule]:
             return False
 
+        rule_exclusions = self.config[rule]["exclusions"]
+        if filename in rule_exclusions.get("files", []):
+            return True
+        if message_id in rule_exclusions.get("messages", []):
+            return True
+
+        return False
+
+    def check_typography(self, node):
+
         # Serialize message without comments
         parts = []
         parts.append(f"{node.id.name} =")
@@ -114,23 +134,26 @@ class Linter(visitor.Visitor):
         cleaned_str = html_stripper.get_data()
 
         if self.apostrophe_re.search(cleaned_str):
-            if not exclude_string("TE01", node):
+            if not self.exclude_message("TE01", node.id.name):
                 self.add_error(
                     node,
+                    node.id.name,
                     "TE01",
                     "Strings with apostrophes should use foo\u2019s instead of foo's.",
                 )
         if self.incorrect_apostrophe_re.search(cleaned_str):
-            if not exclude_string("TE02", node):
+            if not self.exclude_message("TE02", node.id.name):
                 self.add_error(
                     node,
+                    node.id.name,
                     "TE02",
                     "Strings with apostrophes should use foo\u2019s instead of foo\u2018s.",
                 )
         if self.single_quote_re.search(cleaned_str):
-            if not exclude_string("TE03", node):
+            if not self.exclude_message("TE03", node.id.name):
                 self.add_error(
                     node,
+                    node.id.name,
                     "TE03",
                     "Single-quoted strings should use Unicode \u2018foo\u2019 instead of 'foo'.",
                 )
@@ -139,18 +162,20 @@ class Linter(visitor.Visitor):
             for regex in self.ftl_syntax_re:
                 cleaned_str = regex.sub("", cleaned_str)
 
-            if self.double_quote_re.search(cleaned_str) and not exclude_string(
-                "TE04", node
+            if self.double_quote_re.search(cleaned_str) and not self.exclude_message(
+                "TE04", node.id.name
             ):
                 self.add_error(
                     node,
+                    node.id.name,
                     "TE04",
                     'Double-quoted strings should use Unicode \u201cfoo\u201d instead of "foo".',
                 )
         if self.ellipsis_re.search(cleaned_str):
-            if not exclude_string("TE05", node):
+            if not self.exclude_message("TE05", node.id.name):
                 self.add_error(
                     node,
+                    node.id.name,
                     "TE05",
                     "Strings with an ellipsis should use the Unicode \u2026 character"
                     " instead of three periods",
@@ -181,7 +206,7 @@ class Linter(visitor.Visitor):
     def visit_Attribute(self, node):
         # Log errors if attributes are not supported
         if "SY05" in self.config and self.config["SY05"]["disabled"]:
-            self.add_error(node, "SY05", "Attributes are not supported.")
+            self.add_error(node, None, "SY05", "Attributes are not supported.")
             pass
         else:
             # Only visit values for Attribute nodes, the identifier comes from dom.
@@ -193,16 +218,18 @@ class Linter(visitor.Visitor):
         pass
 
     def visit_Term(self, node):
+        self.state["is_term"] = True
         # There must be at least one message or term between group comments.
         self.state["can_have_group_comment"] = True
 
         # Log errors if terms are not supported
         if "SY01" in self.config and self.config["SY01"]["disabled"]:
-            self.add_error(node, "SY01", "Terms are not supported.")
+            self.add_error(node, node.id, "SY01", "Terms are not supported.")
 
         super().generic_visit(node)
 
     def visit_Message(self, node):
+        self.state["is_term"] = False
         # There must be at least one message or term between group comments.
         self.state["can_have_group_comment"] = True
 
@@ -210,6 +237,7 @@ class Linter(visitor.Visitor):
         if node.id.name in self.ids:
             self.add_error(
                 node,
+                node.id.name,
                 "MI01",
                 f"Identifier {node.id.name} is present more than once in the file.",
             )
@@ -219,12 +247,40 @@ class Linter(visitor.Visitor):
         # Check typography
         self.check_typography(node)
 
+        self.message_errors["brands"] = None
+        self.message_errors["banned_words"] = None
         super().generic_visit(node)
 
+        if self.message_errors["brands"] is not None and not self.exclude_message(
+            "CO01", node.id.name, self.path
+        ):
+            self.add_error(
+                # Using the passed node, otherwise the offset would
+                # potentially refer to the comment
+                self.message_errors["brands"]["node"],
+                node.id.name,
+                "CO01",
+                "Strings should use the corresponding terms instead of"
+                f" hard-coded brand names ({self.message_errors['brands']['matches']})",
+            )
+
+        if self.message_errors["banned_words"] is not None and not self.exclude_message(
+            "CO02", node.id.name, self.path
+        ):
+            self.add_error(
+                # Using the passed node, otherwise the offset would
+                # potentially refer to the comment
+                self.message_errors["banned_words"]["node"],
+                node.id.name,
+                "CO02",
+                f"Strings should not include banned words"
+                f" ({self.message_errors['banned_words']['matches']})",
+            )
+
     def visit_MessageReference(self, node):
         # Log errors if message references are not supported
         if "SY02" in self.config and self.config["SY02"]["disabled"]:
-            self.add_error(node, "SY02", "Message references are not supported.")
+            self.add_error(node, None, "SY02", "Message references are not supported.")
 
         # We don't recurse into message references, the identifiers are either
         # checked elsewhere or are attributes and come from DOM.
@@ -234,30 +290,59 @@ class Linter(visitor.Visitor):
     def visit_TermReference(self, node):
         # Log errors if term references are not supported
         if "SY03" in self.config and self.config["SY03"]["disabled"]:
-            self.add_error(node, "SY03", "Terms are not supported.")
+            self.add_error(node, None, "SY03", "Terms are not supported.")
         pass
 
+    def visit_TextElement(self, node):
+        html_stripper = MLStripper()
+        html_stripper.feed(node.value)
+        cleaned_str = html_stripper.get_data()
+
+        found_brands = []
+        for brand in self.brand_names:
+            if brand in cleaned_str:
+                found_brands.append(brand)
+        if found_brands:
+            self.message_errors["brands"] = {
+                "node": node,
+                "matches": ", ".join(found_brands),
+            }
+
+        found_banned_words = []
+        for word in self.banned_words:
+            brand_re = re.compile(r"\b" + word + r"\b")
+            if brand_re.search(cleaned_str.lower()):
+                found_banned_words.append(word)
+        if found_banned_words:
+            self.message_errors["banned_words"] = {
+                "node": node,
+                "matches": ", ".join(found_banned_words),
+            }
+
     def visit_Identifier(self, node):
+        message_id = f"-{node.name}" if self.state["is_term"] else node.name
         if (
             "ID01" in self.config
             and self.config["ID01"]["enabled"]
-            and self.path not in self.config["ID01"]["exclusions"]["files"]
-            and node.name not in self.config["ID01"]["exclusions"]["messages"]
+            and not self.exclude_message("ID01", node.name, self.path)
             and not self.identifier_re.fullmatch(node.name)
         ):
             self.add_error(
-                node, "ID01", "Identifiers may only contain lowercase characters and -"
+                node,
+                message_id,
+                "ID01",
+                "Identifiers may only contain lowercase characters and -",
             )
 
         if (
             "ID02" in self.config
             and self.config["ID02"]["enabled"]
+            and not self.exclude_message("ID02", node.name, self.path)
             and len(node.name) < self.config["ID02"]["min_length"]
-            and self.path not in self.config["ID02"]["exclusions"]["files"]
-            and node.name not in self.config["ID02"]["exclusions"]["messages"]
         ):
             self.add_error(
                 node,
+                message_id,
                 "ID02",
                 f"Identifiers must be at least {self.config['ID02']['min_length']} characters long",
             )
@@ -272,6 +357,7 @@ class Linter(visitor.Visitor):
         if not self.state["node_can_be_resource_comment"]:
             self.add_error(
                 node,
+                None,
                 "RC01",
                 "Resource comments (###) should be placed at the top of the file, just "
                 "after the license header. There should only be one resource comment "
@@ -289,6 +375,7 @@ class Linter(visitor.Visitor):
         if lines_after != 2:
             self.add_error(
                 node,
+                None,
                 "RC02",
                 "Resource comments (###) should be followed by one empty line.",
             )
@@ -297,6 +384,7 @@ class Linter(visitor.Visitor):
         if lines_before != 2:
             self.add_error(
                 node,
+                None,
                 "RC03",
                 "Resource comments (###) should have one empty line above them.",
             )
@@ -305,7 +393,7 @@ class Linter(visitor.Visitor):
     def visit_SelectExpression(self, node):
         # Log errors if variants are not supported
         if node.variants and "SY04" in self.config and self.config["SY04"]["disabled"]:
-            self.add_error(node, "SY04", "Variants are not supported.")
+            self.add_error(node, None, "SY04", "Variants are not supported.")
             pass
         else:
             # We only want to visit the variant values, the identifiers in selectors
@@ -323,6 +411,7 @@ class Linter(visitor.Visitor):
         if not self.state["can_have_group_comment"]:
             self.add_error(
                 node,
+                None,
                 "GC04",
                 "Group comments (##) must be followed by at least one message. Make sure "
                 "that a single group comment with multiple pararaphs is not separated by "
@@ -344,6 +433,7 @@ class Linter(visitor.Visitor):
 
             self.add_error(
                 node,
+                None,
                 "GC01",
                 "Group comments (##) should not be at the end of the file, they should "
                 "always be above a message. Only an empty group comment is allowed at "
@@ -354,6 +444,7 @@ class Linter(visitor.Visitor):
         if lines_after != 2:
             self.add_error(
                 node,
+                None,
                 "GC02",
                 "Group comments (##) should be followed by one empty line.",
             )
@@ -362,6 +453,7 @@ class Linter(visitor.Visitor):
         if lines_before != 2:
             self.add_error(
                 node,
+                None,
                 "GC03",
                 "Group comments (##) should have an empty line before them.",
             )
@@ -373,16 +465,18 @@ class Linter(visitor.Visitor):
 
         # Log errors if variable references are not supported
         if "SY06" in self.config and self.config["SY06"]["disabled"]:
-            self.add_error(node, "SY06", "Variable references are not supported.")
+            self.add_error(node, None, "SY06", "Variable references are not supported.")
 
         pass
 
-    def add_error(self, node, rule, msg):
+    def add_error(self, node, message_id, rule, msg):
         (col, line) = self.span_to_line_and_col(node.span)
 
         file_path = os.path.relpath(self.path, self.root_folder)
+        message_id = message_id if message_id is not None else "-"
         error_msg = f"""
             File path: {file_path}
+            Message ID: {message_id}
             Position: line {line} column {col}
             Error ({rule}): {msg}"""
 
@@ -475,13 +569,15 @@ def lint(file_paths, config_path):
         for path in paths:
             # Ensure that the file has an empty line at the end
             with open(path, "r", encoding="utf-8") as f:
-                last_line = f.readlines()[-1]
-                if last_line == last_line.rstrip():
-                    rel_path = os.path.relpath(path, root_folder)
-                    error_msg = f"""
+                file_content = f.readlines()
+                if len(file_content) > 0:
+                    last_line = file_content[-1]
+                    if last_line == last_line.rstrip():
+                        rel_path = os.path.relpath(path, root_folder)
+                        error_msg = f"""
             File path: {rel_path}
             Error (MI02): Missing empty line at the end of the file"""
-                    results.append(error_msg)
+                        results.append(error_msg)
 
                 # Reset position after reading the whole content and lint
                 f.seek(0)
